﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class SongManager : MonoBehaviour {

    private int currentBPM;
    public Song song;
    public Camera camera;
    public AudioSource audioSong;
    public CharSystem charSystem;

    public float timeBufferSec;

    private int index;
    private int temps;

    private float dernierTempsNote;
    private float dernierTempsBeat;
    
    private List<Direction> notes;
    private List<float> notesTime;
    private List<bool> notesPlayed;

    private bool enPause;

    // Use this for initialization
    void Start () {
        index = 0;
        currentBPM = 0;
        dernierTempsNote = 0f;
        dernierTempsBeat = 0f;

        enPause = true;

        notes = new List<Direction>();
        notesTime = new List<float>();
        notesPlayed = new List<bool>();

        setNextNote(song.Track[index]);
        
        StartCoroutine("PlayTrack");
        StartCoroutine("PlayBeat");
    }

    public void Pause(bool mettreEnPause)
    {
        enPause = mettreEnPause;
        if (enPause)
            audioSong.Pause();
        else
        {
            if (audioSong.isPlaying)
                audioSong.UnPause();
            else
                audioSong.Play();
        }
    }

    public void Restart()
    {
        StopAllCoroutines();
        enPause = false;
        audioSong.time = 0f;

        index = 0;
        currentBPM = 0;
        dernierTempsNote = 0f;
        dernierTempsBeat = 0f;

        notes = new List<Direction>();
        notesTime = new List<float>();
        notesPlayed = new List<bool>();

        setNextNote(song.Track[index]);

        StartCoroutine("PlayTrack");
        StartCoroutine("PlayBeat");
    }

    IEnumerator PlayBeat()
    {
        float diff = 0f;
        while (true)
        {
            while (enPause)
            {
                yield return null;
            }

            float tempsProchaine = dernierTempsBeat + getBeat();
            charSystem.BeatNow();
            dernierTempsBeat = tempsProchaine;
            while (audioSong.time < tempsProchaine)
            {
                while (enPause)
                {
                    yield return null;
                }

                diff = tempsProchaine - audioSong.time;
                if (diff < 0)
                    break;

                while (enPause)
                {
                    yield return null;
                }
                yield return new WaitForSeconds((diff) / 2);
            }
        }
    }

    IEnumerator PlayTrack()
    {
        while (enPause)
        {
            yield return null;
        }

        audioSong.Play();
        float diff = 0;
        while (song.Track.Count > index)
        {
            while (enPause)
            {
                yield return null;
            }

            float tempsProchaine = dernierTempsNote + PlayWave(song.Track[index]);
            dernierTempsNote = tempsProchaine;
            while (audioSong.time < tempsProchaine)
            {
                while (enPause)
                {
                    yield return null;
                }
                clearLists();
                diff = tempsProchaine - audioSong.time;
                if (diff < 0)
                    break;

                while (enPause)
                {
                    yield return null;
                }
                yield return new WaitForSeconds((diff) / 2);
            }

            index++;
        }
    }

    float getBeat()
    {
        return 60f / currentBPM;
    }

    float PlayWave(Wave wave)
    {
        currentBPM = wave.Bpm;
        clearLists();
        float timeBeforeNext = getTimeBeforeNext(wave);
        if (song.Track.Count > index + 1)
            setNextNote(song.Track[index]);
        charSystem.DoCharacterWave(EnumActionTurn.ai, wave.Direction, timeBeforeNext);
        return timeBeforeNext;
    }

    float getTimeBeforeNext(Wave wave)
    {
        return wave.Temps * getBeat();
    }

    void setNextNote(Wave wave)
    {
        if(wave.IsUs)
        {
            float nextTime = dernierTempsNote + getTimeBeforeNext(wave);
            notes.Add(wave.Direction);
            notesTime.Add(nextTime);
            notesPlayed.Add(false);
        }
    }

    public void PlayerSendNote(Direction dir)
    {
        clearLists();
        ValidateNote(dir);
        charSystem.DoCharacterWave(EnumActionTurn.player, dir);
    }

    void ValidateNote(Direction dir)
    {
        if(notes.Count > 0)
        {
            for(int i = 0; i < notes.Count; i++)
            {
                float avant = notesTime[i] - timeBufferSec;
                float apres = notesTime[i] + timeBufferSec;
                float now = audioSong.time;
                
                if(now <= apres && now >= avant)
                {
                    if(!notesPlayed[i])
                    {
                        notesPlayed[i] = true;
                        if(notes[i] == dir)
                        {
                            //TODO : +25 pts !!!
                            charSystem.DoCharacterWave(EnumActionTurn.player, dir);
                            Debug.Log("25");
                        }
                        else
                        {
                            charSystem.DoCharacterWave(EnumActionTurn.player, Direction.FAIL);
                            Debug.Log("FAIL 1");
                        }
                    }
                }
            }
        }
    }

    void clearLists()
    {
        for (int i = notes.Count - 1; i >= 0; i--)
        {
            float apres = notesTime[i] + timeBufferSec;
            float now = audioSong.time;

            if (now > apres)
            {
                if(!notesPlayed[i])
                {
                    charSystem.DoCharacterWave(EnumActionTurn.player, Direction.FAIL);
                    Debug.Log("FAIL 2");
                }
                notes.RemoveAt(i);
                notesPlayed.RemoveAt(i);
                notesTime.RemoveAt(i);
            }
        }
    }
}
